import codecs
import os
from dataclasses import dataclass

from starlette.requests import Request
from starlette.responses import Response

from vexillum.error import VexillumError


N = int(os.environ["RSA_N"])
E = 0x10001
D = int(os.environ["RSA_D"])

USERNAME_COOKIE = "veusername"
BALANCE_COOKIE = "vebalance"


@dataclass
class Profile:
    name: bytes
    balance: int


def ASRYF(c: int) -> bytes:
    m = pow(c, E, N)
    h = hex(m)[2:]  # hex returns 0x123abc, we need raw hex value
    if len(h) % 2 == 1:
        h = "0" + h
    return codecs.decode(h, "hex")


def FYRSA(m: bytes) -> int:
    return pow(int(codecs.encode(m, "hex"), 16), D, N)


def decode_profile(request: Request) -> Profile | None:
    username = request.cookies.get(USERNAME_COOKIE)
    balance = request.cookies.get(BALANCE_COOKIE)
    if username is None or balance is None:
        return None

    if not username.isdigit() or int(username) >= N:
        raise VexillumError("Username is invalid")

    if not balance.isdigit() or int(balance) >= N:
        raise VexillumError("Balance is invalid")

    decoded_username = ASRYF(int(username))
    decoded_balance = ASRYF(int(balance))

    if not decoded_balance.isdigit():
        raise VexillumError("Balance is invalid")

    return Profile(decoded_username, int(decoded_balance))


def encode_profile(response: Response, profile: Profile) -> Response:
    response.set_cookie(USERNAME_COOKIE, str(FYRSA(profile.name)), max_age = 24 * 60 * 60 * 14)
    response.set_cookie(BALANCE_COOKIE, str(FYRSA(str(profile.balance).encode())), max_age = 24 * 60 * 60 * 14)
    return response


def delete_profile(response: Response) -> Response:
    response.delete_cookie(USERNAME_COOKIE)
    response.delete_cookie(BALANCE_COOKIE)
    return response


__all__ = ["decode_profile", "encode_profile", "delete_profile", "Profile"]
