from starlette.requests import Request
from starlette.responses import RedirectResponse
from starlette.types import ASGIApp, Receive, Scope, Send

from vexillum.cryptography import decode_profile


class AuthorizationMiddleware:
    def __init__(self, app: ASGIApp) -> None:
        self.app = app

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        if scope["type"] == "http":
            request = Request(scope)
            
            profile = decode_profile(request)
            if profile is None:
                return await RedirectResponse("/subsigno", status_code=303)(scope, receive, send)

            scope["profile"] = profile
        
        await self.app(scope, receive, send)


__all__ = ["AuthorizationMiddleware"]
