from starlette.requests import Request
from starlette.responses import RedirectResponse, Response
from starlette_wtf import StarletteForm, csrf_protect
from wtforms.fields import StringField
from wtforms.validators import InputRequired, Regexp, Length

from vexillum import cryptography
from vexillum.templates import templates


class RegistrationForm(StarletteForm):
    name = StringField("Name", validators=[InputRequired(message="Malus nomen"), Regexp("^[A-Za-z ]+$", message="Malus nomen"), Length(max=32, message="Malus nomen")])


@csrf_protect
async def register(request: Request) -> Response:
    form = await RegistrationForm.from_formdata(request)

    if await form.validate_on_submit():
        profile = cryptography.Profile(form.name.data.encode(), 10)  # type: ignore
        return cryptography.encode_profile(RedirectResponse("/", status_code=303), profile)

    return templates.TemplateResponse(request, "register.html", {"form": form})


__all__ = ["register"]
